import type {ClientPerspective, ContentSourceMap, QueryParams} from '@sanity/client'
import {ResponseQueryOptions, SanityClient} from '@sanity/client'
import type {ResolveStudioUrl, StudioUrl} from '@sanity/client/csm'
import {SanityStegaClient} from '@sanity/client/stega'
import type {
  createQueryStore as createQueryStore_2,
  EnableLiveModeOptions,
} from '@sanity/core-loader'
import {CreateQueryStoreOptions, QueryStoreState} from '@sanity/core-loader'
import type {EncodeDataAttributeFunction} from '@sanity/core-loader/encode-data-attribute'

export declare const createQueryStore: (options: CreateQueryStoreOptions) => QueryStore

export declare const loadQuery: <QueryResponseResult>(
  query: string,
  params?: QueryParams,
  options?: Pick<
    ResponseQueryOptions,
    'perspective' | 'cache' | 'next' | 'useCdn' | 'stega' | 'tag' | 'headers'
  >,
) => Promise<QueryResponseInitial<QueryResponseResult>>

export declare type NonUndefinedGuard<T> = T extends undefined ? never : T

export declare interface QueryResponseInitial<QueryResponseResult> {
  data: QueryResponseResult
  sourceMap: ContentSourceMap | undefined
  /**
   * The perspective used to fetch the data, if not provided it'll assume 'published'
   */
  perspective?: ClientPerspective
}

declare interface QueryStore {
  loadQuery: <QueryResponseResult>(
    query: string,
    params?: QueryParams,
    options?: Pick<
      ResponseQueryOptions,
      'perspective' | 'cache' | 'next' | 'useCdn' | 'stega' | 'tag' | 'headers'
    >,
  ) => Promise<QueryResponseInitial<QueryResponseResult>>
  setServerClient: ReturnType<typeof createQueryStore_2>['setServerClient']
  useQuery: {
    <QueryResponseResult = unknown, QueryResponseError = unknown>(
      query: string,
      params?: QueryParams,
      options?: UseQueryOptionsUndefinedInitial,
    ): QueryStoreState<QueryResponseResult, QueryResponseError> & WithEncodeDataAttribute
    <QueryResponseResult = unknown, QueryResponseError = unknown>(
      query: string,
      params?: QueryParams,
      options?: UseQueryOptionsDefinedInitial<QueryResponseResult>,
    ): Omit<QueryStoreState<QueryResponseResult, QueryResponseError>, 'data'> & {
      data: QueryResponseResult
    } & WithEncodeDataAttribute
  }
  useLiveMode: UseLiveModeHook
}
export {QueryStore as BaseQueryStore}
export {QueryStore}

export declare const setServerClient: (client: SanityClient | SanityStegaClient) => void

export declare const useEncodeDataAttribute: () => void

export declare const useLiveMode: UseLiveModeHook

export declare type UseLiveModeHook = (
  options: EnableLiveModeOptions & {
    /**
     * Set this option to activate `encodeDataAttribute` on `useQuery` hooks when stega isn't used.
     */
    studioUrl?: StudioUrl | ResolveStudioUrl | undefined
  },
) => void

export declare const useQuery: {
  <QueryResponseResult = unknown, QueryResponseError = unknown>(
    query: string,
    params?: QueryParams,
    options?: UseQueryOptionsUndefinedInitial,
  ): QueryStoreState<QueryResponseResult, QueryResponseError> & WithEncodeDataAttribute
  <QueryResponseResult = unknown, QueryResponseError = unknown>(
    query: string,
    params?: QueryParams,
    options?: UseQueryOptionsDefinedInitial<QueryResponseResult>,
  ): Omit<QueryStoreState<QueryResponseResult, QueryResponseError>, 'data'> & {
    data: QueryResponseResult
  } & WithEncodeDataAttribute
}

export declare type UseQueryHook = <QueryResponseResult = unknown, QueryResponseError = unknown>(
  query: string,
  params?: QueryParams,
  options?: UseQueryOptions<QueryResponseResult>,
) => QueryStoreState<QueryResponseResult, QueryResponseError> & WithEncodeDataAttribute

export declare interface UseQueryOptions<QueryResponseResult = unknown> {
  /**
   * Initial `data` and `sourceMap`, used with SSR hydration and is required if `ssr: true`
   * and an optional speed optimization if `ssr: false`.
   * It's recommended to set `initial` to the return value of `loadQuery()`.
   * @example
   * ```ts
   * const query = `*[_type == "author" && slug.current == $slug][0]`
   * export const getServerSideProps = async ({params}) => {
   *   const initial = await loadQuery<AuhthorType>(query, params)
   *   return { props: { params, initial } }
   * }
   * export default function Page({params, initial}) {
   *   const {data} = useQuery<AuthorType>(query, params, {initial})
   * }
   * ```
   */
  initial?: QueryResponseInitial<QueryResponseResult>
}

export declare interface UseQueryOptionsDefinedInitial<QueryResponseResult = unknown> {
  /**
   * Initial `data` and `sourceMap`, used with SSR hydration and is required if `ssr: true`
   * and an optional speed optimization if `ssr: false`.
   * It's recommended to set `initial` to the return value of `loadQuery()`.
   * @example
   * ```ts
   * const query = `*[_type == "author" && slug.current == $slug][0]`
   * export const getServerSideProps = async ({params}) => {
   *   const initial = await loadQuery<AuhthorType>(query, params)
   *   return { props: { params, initial } }
   * }
   * export default function Page({params, initial}) {
   *   const {data} = useQuery<AuthorType>(query, params, {initial})
   * }
   * ```
   */
  initial: NonUndefinedGuard<QueryResponseInitial<QueryResponseResult>>
}

export declare interface UseQueryOptionsUndefinedInitial {
  /**
   * Initial `data` and `sourceMap`, used with SSR hydration and is required if `ssr: true`
   * and an optional speed optimization if `ssr: false`.
   * It's recommended to set `initial` to the return value of `loadQuery()`.
   * @example
   * ```ts
   * const query = `*[_type == "author" && slug.current == $slug][0]`
   * export const getServerSideProps = async ({params}) => {
   *   const initial = await loadQuery<AuhthorType>(query, params)
   *   return { props: { params, initial } }
   * }
   * export default function Page({params, initial}) {
   *   const {data} = useQuery<AuthorType>(query, params, {initial})
   * }
   * ```
   */
  initial?: undefined
}

export declare type WithEncodeDataAttribute = {
  encodeDataAttribute: EncodeDataAttributeFunction
}

export * from '@sanity/core-loader'

export {}
